@@ -16,77 +16,62 @@
 # A must be symmetric and positive semi definite, but not necessarily
 # normalized in any specific way.
 #' @import Matrix
-laplacian_eigenmap <- function(A, ndim = 2, verbose = FALSE) {
+laplacian_eigenmap <- function(A, ndim = 2, verbose = FALSE, force_irlba = FALSE) {
+  if (rspectra_is_installed() && !force_irlba) {
+    coords <- rspectra_laplacian_eigenmap(A, ndim, verbose = verbose)
+  } else {
+    coords <- irlba_laplacian_eigenmap(A, ndim, verbose = verbose)
+  }
+  coords
+}
+
+rspectra_laplacian_eigenmap <- function(A, ndim = 2, verbose = FALSE) {
   if (nrow(A) < 3) {
     tsmessage("Graph too small, using random initialization instead")
     return(rand_init(nrow(A), ndim))
   }
 
-  tsmessage("Initializing from Laplacian Eigenmap")
+  tsmessage("Initializing from Laplacian Eigenmap (via RSpectra)")
   # Equivalent to: D <- diag(colSums(A)); M <- solve(D) %*% A
   # This effectively row-normalizes A: colSums is normally faster than rowSums
   # and because A is symmetric, they're equivalent
   M <- A / colSums(A)
-  connected <- connected_components(M)
-  if (connected$n_components > 1) {
-    tsmessage(
-      "Found ", connected$n_components, " connected components, ",
-      "initializing each component separately"
-    )
-    fn_name <- as.character(match.call()[[1]])
-    return(subgraph_init(fn_name, connected,
-      A = A, ndim = ndim,
-      verbose = verbose
-    ))
-  }
-
-  res <- NULL
-  k <- ndim + 1
-  n <- nrow(M)
-  suppressWarnings(
-    res <- tryCatch(RSpectra::eigs(M,
-      k = k, which = "LM",
-      opt = list(tol = 1e-4)
-    ),
-    error = function(c) {
-      NULL
-    }
-    )
-  )
-
+  res <- rspectra_eigs_asym(M, ndim)
   if (is.null(res) || ncol(res$vectors) < ndim) {
     message(
       "Laplacian Eigenmap failed to converge, ",
       "using random initialization instead"
     )
+    n <- nrow(M)
     return(rand_init(n, ndim))
   }
 
-  vecs <- as.matrix(res$vectors[, 2:(ndim + 1)])
-  Re(vecs)
+  # return the smallest eigenvalues
+  as.matrix(Re(res$vectors[, 2:(ndim + 1)]))
 }
 
-# Use a normalized Laplacian.
-normalized_laplacian_init <- function(A, ndim = 2, verbose = FALSE) {
+irlba_laplacian_eigenmap <- function(A, ndim = 2, verbose = FALSE) {
   if (nrow(A) < 3) {
     tsmessage("Graph too small, using random initialization instead")
     return(rand_init(nrow(A), ndim))
   }
-  tsmessage("Initializing from normalized Laplacian")
-  connected <- connected_components(A)
-  if (connected$n_components > 1) {
-    tsmessage(
-      "Found ", connected$n_components, " connected components, ",
-      "initializing each component separately"
-    )
-    fn_name <- as.character(match.call()[[1]])
-    return(subgraph_init(fn_name, connected,
-      A = A, ndim = ndim,
-      verbose = verbose
-    ))
-  }
+  tsmessage("Initializing from Laplacian Eigenmap (via irlba)")
 
-  n <- nrow(A)
+  lapA <- form_modified_laplacian(A, ret_d = TRUE)
+  res <- irlba_spectral_tsvd(lapA$L, ndim + 1)
+  if (is.null(res) || ncol(res$vectors) < ndim || !res$converged) {
+    message(
+      "Laplacian Eigenmap failed to converge, ",
+      "using random initialization instead"
+    )
+    return(rand_init(nrow(A), ndim))
+  }
+  res <- lapA$Disqrt * res$vectors[, 2:(ndim + 1), drop = FALSE]
+  # re-scale the vectors to length 1
+  sweep(res, 2, sqrt(colSums(res * res)), `/`)
+}
+
+form_normalized_laplacian <- function(A) {
   # Normalized Laplacian: clear and close to UMAP code, but very slow in R
   # I <- diag(1, nrow = n, ncol = n)
   # D <- diag(1 / sqrt(colSums(A)))
@@ -96,37 +81,125 @@ normalized_laplacian_init <- function(A, ndim = 2, verbose = FALSE) {
   Dsq <- sqrt(Matrix::colSums(A))
   L <- -Matrix::t(A / Dsq) / Dsq
   Matrix::diag(L) <- 1 + Matrix::diag(L)
+  L
+}
 
-  k <- ndim + 1
-  opt <- list(tol = 1e-4)
-  suppressWarnings(
-    res <- tryCatch(RSpectra::eigs_sym(L, k = k, which = "SM", opt = opt),
-      error = function(c) {
-        NULL
-      }
-    )
-  )
-  if (is.null(res) || ncol(res$vectors) < ndim) {
-    suppressWarnings(
-      res <- tryCatch(RSpectra::eigs_sym(L,
-        k = k, which = "LM", sigma = 0,
-        opt = opt
-      ),
-      error = function(c) {
-        NULL
-      }
-      )
-    )
-    if (is.null(res) || ncol(res$vectors) < ndim) {
-      message(
-        "Spectral initialization failed to converge, ",
-        "using random initialization instead"
-      )
-      return(rand_init(n, ndim))
-    }
+# The symmetrized graph Laplacian (Lsym) but shifted so that:
+# the bottom eigenvectors of Lsym correspond to the top singular vectors of
+# this matrix (hence can be used with truncated SVD), and the eigenvalues
+# are all positive, so we don't lose sign and hence correct eigenvector ordering
+# when using the singular values (lambda = 2 - d)
+form_modified_laplacian <- function(A, ret_d = FALSE) {
+  Dsq <- sqrt(Matrix::colSums(A))
+  L <- Matrix::t(A / Dsq) / Dsq
+  Matrix::diag(L) <- 1 + Matrix::diag(L)
+  if (ret_d) {
+    list(L = L, Disqrt = 1 / Dsq)
+  } else {
+    L
   }
-  vec_indices <- rev(order(res$values, decreasing = TRUE)[1:ndim])
-  as.matrix(Re(res$vectors[, vec_indices]))
+}
+
+# Return the ndim eigenvectors associated with the ndim largest eigenvalues
+sort_eigenvectors <- function(eig_res, ndim) {
+  vec_indices <- rev(order(eig_res$values, decreasing = TRUE)[1:ndim])
+  as.matrix(Re(eig_res$vectors[, vec_indices]))
+}
+
+normalized_laplacian_init <- function(A, ndim = 2, verbose = FALSE, force_irlba = FALSE) {
+  if (rspectra_is_installed() && !force_irlba) {
+    coords <- rspectra_normalized_laplacian_init(A, ndim, verbose = verbose)
+  } else {
+    coords <- irlba_normalized_laplacian_init(A, ndim, verbose = verbose)
+  }
+  coords
+}
+
+rspectra_normalized_laplacian_init <- function(A, ndim = 2, verbose = FALSE) {
+  if (nrow(A) < 3) {
+    tsmessage("Graph too small, using random initialization instead")
+    return(rand_init(nrow(A), ndim))
+  }
+  tsmessage("Initializing from normalized Laplacian")
+
+  L <- form_normalized_laplacian(A)
+  res <- rspectra_eigs_sym(L, ndim, verbose = verbose)
+
+  if (is.null(res) || ncol(res$vectors) < ndim) {
+    message(
+      "Spectral initialization failed to converge, ",
+      "using random initialization instead"
+    )
+    n <- nrow(A)
+    return(rand_init(n, ndim))
+  }
+  sort_eigenvectors(res, ndim)
+}
+
+# Use a normalized Laplacian and use truncated SVD
+irlba_tsvd_normalized_laplacian_init <- function(A, ndim = 2, verbose = FALSE) {
+  if (nrow(A) < 3) {
+    tsmessage("Graph too small, using random initialization instead")
+    return(rand_init(nrow(A), ndim))
+  }
+  tsmessage("Initializing from normalized Laplacian")
+
+  L <- form_modified_laplacian(A)
+  res <- irlba_spectral_tsvd(L, ndim + 1)
+  if (is.null(res) || ncol(res$vectors) < ndim || !res$converged) {
+    message(
+      "Spectral initialization failed to converge, ",
+      "using random initialization instead"
+    )
+    n <- nrow(A)
+    return(rand_init(n, ndim))
+  }
+  res$vectors[, 2:(ndim + 1), drop = FALSE]
+}
+
+irlba_spectral_tsvd <- function(L, n, iters = 1000) {
+  suppressWarnings(res <- irlba::irlba(L, nv = n, nu = 0, maxit = iters))
+  list(vectors = res$v, values = 2.0 - res$d, converged = res$iter != iters)
+}
+
+irlba_eigs_asym <- function(L, ndim) {
+  suppressWarnings(res <- tryCatch(
+    {
+      res <- irlba::partial_eigen(
+        L,
+        n = ndim + 1,
+        symmetric = FALSE,
+        smallest = TRUE,
+        tol = 1e-3,
+        maxit = 1000,
+        verbose = TRUE
+      )
+      res$values <- sqrt(res$values)
+      res
+    },
+    error = function(c) {
+      NULL
+    }
+  ))
+  res
+}
+
+irlba_eigs_sym <- function(L, ndim, smallest = TRUE) {
+  suppressWarnings(res <- tryCatch(
+    res <- irlba::partial_eigen(
+      L,
+      n = ndim + 1,
+      symmetric = TRUE,
+      smallest = smallest,
+      tol = 1e-3,
+      maxit = 1000,
+      verbose = FALSE
+    ),
+    error = function(c) {
+      NULL
+    }
+  ))
+  res
 }
 
 # Use irlba's partial_eigen instead of RSpectra
@@ -137,61 +210,40 @@ irlba_normalized_laplacian_init <- function(A, ndim = 2, verbose = FALSE) {
   }
   tsmessage("Initializing from normalized Laplacian (using irlba)")
 
-  n <- nrow(A)
-  Dsq <- sqrt(Matrix::colSums(A))
-  L <- -Matrix::t(A / Dsq) / Dsq
-  Matrix::diag(L) <- 1 + Matrix::diag(L)
-
-  k <- ndim + 1
-
-  suppressWarnings(
-    res <- tryCatch(res <- irlba::partial_eigen(L,
-      n = k, symmetric = TRUE,
-      smallest = TRUE, tol = 1e-3,
-      maxit = 1000, verbose = TRUE
-    ),
-    error = function(c) {
-      NULL
-    }
-    )
-  )
+  # Using the normalized Laplacian and looking for smallest eigenvalues does
+  # not work well with irlba's partial_eigen routine, so form the shifted
+  # Laplacian and look for largest eigenvalues
+  L <- form_modified_laplacian(A)
+  res <- irlba_eigs_sym(L, ndim, smallest = FALSE)
+  # shift back the eigenvalues
+  res$values <- 2.0 - res$values
   if (is.null(res) || ncol(res$vectors) < ndim) {
     message(
       "Spectral initialization failed to converge, ",
       "using random initialization instead"
     )
+    n <- nrow(A)
     return(rand_init(n, ndim))
   }
-  vec_indices <- rev(order(res$values, decreasing = TRUE)[1:ndim])
-  as.matrix(Re(res$vectors[, vec_indices]))
+  sort_eigenvectors(res, ndim)
 }
 
 
 # Default UMAP initialization
 # spectral decomposition of the normalized Laplacian + some noise
-spectral_init <- function(A, ndim = 2, verbose = FALSE) {
+spectral_init <- function(A, ndim = 2, verbose = FALSE, force_irlba = FALSE) {
   if (nrow(A) < 3) {
     tsmessage("Graph too small, using random initialization instead")
     return(rand_init(nrow(A), ndim))
   }
-  tsmessage("Initializing from normalized Laplacian + noise")
-  connected <- connected_components(A)
-  if (connected$n_components > 1) {
-    tsmessage(
-      "Found ", connected$n_components, " connected components, ",
-      "initializing each component separately"
-    )
-    fn_name <- as.character(match.call()[[1]])
-    return(subgraph_init(fn_name, connected,
-      A = A, ndim = ndim,
-      verbose = verbose
-    ))
+  if (rspectra_is_installed() && !force_irlba) {
+    tsmessage("Initializing from normalized Laplacian + noise (using RSpectra)")
+    coords <- rspectra_normalized_laplacian_init(A, ndim, verbose = FALSE)
+  } else {
+    tsmessage("Initializing from normalized Laplacian + noise (using irlba)")
+    coords <- irlba_tsvd_normalized_laplacian_init(A, ndim, verbose = FALSE)
   }
-  coords <- normalized_laplacian_init(A, ndim, verbose = FALSE)
-  expansion <- 10.0 / max(abs(coords))
-  (coords * expansion) + matrix(stats::rnorm(n = prod(dim(coords)), sd = 0.0001),
-    ncol = ndim
-  )
+  scale_and_jitter(coords, max_coord = 10.0, sd = 0.0001)
 }
 
 irlba_spectral_init <- function(A, ndim = 2, verbose = FALSE) {
@@ -202,35 +254,19 @@ irlba_spectral_init <- function(A, ndim = 2, verbose = FALSE) {
   tsmessage("Initializing from normalized Laplacian (using irlba) + noise")
 
   coords <- irlba_normalized_laplacian_init(A, ndim, verbose = FALSE)
-  expansion <- 10.0 / max(coords)
-  (coords * expansion) + matrix(stats::rnorm(n = prod(dim(coords)), sd = 0.001),
-    ncol = ndim
+  scale_and_jitter(coords, max_coord = 10.0, sd = 0.0001)
+}
+
+# Scales coords so that the largest absolute coordinate is 10.0 then jitters by
+# adding gaussian noise with mean 0 and standard deviation sd
+scale_and_jitter <- function(coords, max_coord = 10.0, sd = 0.0001) {
+  expansion <- 10.0 / max(abs(coords))
+  (coords * expansion) + matrix(stats::rnorm(n = prod(dim(coords)), sd = sd),
+    ncol = ncol(coords)
   )
 }
 
-# Recursively calls the spectral initialization function named fn_name
-# for each subgraph specified by connected
-subgraph_init <- function(fn_name, connected, A, ndim = 2, verbose = FALSE) {
-  init <- NULL
-  for (i in 1:connected$n_components) {
-    subg_idx <- connected$labels == i - 1
-    subg <- A[subg_idx, subg_idx]
-    tsmessage("Initializing subcomponent of size ", nrow(subg))
-    init_conn <- do.call(fn_name, list(
-      A = subg, ndim = ndim,
-      verbose = verbose
-    ))
-    if (is.null(init)) {
-      init <- init_conn
-    }
-    else {
-      init <- rbind(init, init_conn)
-    }
-  }
-  init
-}
-
-# Return the number of connected components in a graph (respresented as a
+# Return the number of connected components in a graph (represented as a
 # sparse matrix).
 connected_components <- function(X) {
   Xt <- Matrix::t(X)
@@ -252,31 +288,36 @@ rand_init_lv <- function(n, ndim, verbose = FALSE) {
 # Rescale embedding so that the standard deviation is the specified value.
 # Default gives initialization like t-SNE, but not random. Large initial
 # distances lead to small gradients, and hence small updates, so should be
-# avoided
-shrink_coords <- function(X, sdev = 1e-4) {
-  scale(X, scale = apply(X, 2, stats::sd) / sdev)
+# avoided.
+scale_coords <- function(X, sdev = 1e-4, verbose = FALSE) {
+  if (is.character(sdev) && sdev == "range") {
+    # #99: range scale coordinates like python UMAP does
+    tsmessage("Range-scaling initial input columns to 0-10")
+    return(apply(X, 2, range_scale, max = 10.0))
+  }
+
+  if (is.null(sdev)) {
+    return(X)
+  }
+  tsmessage("Scaling init to sdev = ", sdev)
+  scale_factor <- apply(X, 2, stats::sd)
+  scale(X, scale = scale_factor / sdev)
 }
 
 # PCA
-pca_init <- function(X, ndim = 2, center = TRUE, verbose = FALSE) {
-  tsmessage("Initializing from PCA")
-  pca_scores(X, ncol = ndim, center = center, verbose = verbose)
-}
-
-
-# Calculates a matrix containing the first ncol columns of the PCA scores.
+# Calculates a matrix containing the first ndim columns of the PCA scores.
 # Returns the score matrix unless ret_extra is TRUE, in which case a list
 # is returned also containing the eigenvalues
-pca_scores <- function(X, ncol = min(dim(X)), center = TRUE, ret_extra = FALSE,
-                       verbose = FALSE) {
-  if (methods::is(X, "dist")) {
-    res_mds <- stats::cmdscale(X, x.ret = TRUE, eig = TRUE, k = ncol)
-
+pca_init <- function(X, ndim = min(dim(X)), center = TRUE, ret_extra = FALSE,
+                     pca_method = "auto", verbose = FALSE) {
+  if (inherits(X, "dist")) {
+    res_mds <- stats::cmdscale(X, x.ret = TRUE, eig = TRUE, k = ndim)
     if (ret_extra || verbose) {
       lambda <- res_mds$eig
-      varex <- sum(lambda[1:ncol]) / sum(lambda)
+      lambda[lambda < 0] <- 0
+      varex <- sum(lambda[1:ndim]) / sum(lambda)
       tsmessage(
-        "PCA (using classical MDS): ", ncol, " components explained ",
+        "PCA (using classical MDS): ", ndim, " components explained ",
         formatC(varex * 100), "% variance"
       )
     }
@@ -285,15 +326,41 @@ pca_scores <- function(X, ncol = min(dim(X)), center = TRUE, ret_extra = FALSE,
   }
 
   # irlba warns about using too large a percentage of total singular value
-  # so don't use if dataset is small compared to ncol
-  if (ncol < 0.5 * min(dim(X))) {
-    return(irlba_scores(X,
-      ncol = ncol, center = center, ret_extra = ret_extra,
-      verbose = verbose
-    ))
+  # so don't use if dataset is small compared to ndim
+  if (pca_method == "auto") {
+    if (ndim < 0.5 * min(dim(X))) {
+      pca_method <- "irlba"
+    } else {
+      pca_method <- "svd"
+    }
   }
 
-  svd_scores(X = X, ncol = ncol, center = center, ret_extra = ret_extra, verbose = verbose)
+  if (pca_method == "bigstatsr") {
+    if (!bigstatsr_is_installed()) {
+      warning(
+        "PCA via bigstatsr requires the 'bigstatsr' package. ",
+        "Please install it. Falling back to 'irlba'"
+      )
+      pca_method <- "irlba"
+    }
+  }
+
+  tsmessage("Using '", pca_method, "' for PCA")
+  pca_fun <- switch(pca_method,
+    irlba = irlba_scores,
+    svdr = irlba_svdr_scores,
+    svd = svd_scores,
+    bigstatsr = bigstatsr_scores,
+    stop("BUG: unknown svd method '", pca_method, "'")
+  )
+
+  do.call(pca_fun, list(
+    X = X,
+    ncol = ndim,
+    center = center,
+    ret_extra = ret_extra,
+    verbose = verbose
+  ))
 }
 
 # Get scores by SVD
@@ -329,43 +396,168 @@ svd_scores <- function(X, ncol = min(dim(X)), center = TRUE, ret_extra = FALSE,
       rotation = rotation,
       center = xcenter
     )
-  }
-  else {
+  } else {
     scores
   }
 }
 
 # Get PCA scores via irlba
 irlba_scores <- function(X, ncol, center = TRUE, ret_extra = FALSE, verbose = FALSE) {
+  if (is.logical(X)) {
+    tsmessage("Converting logical input to numeric for PCA initialization")
+    # convert logical matrix to numeric
+    X <- X * 1
+  }
   res <- irlba::prcomp_irlba(X,
     n = ncol, retx = TRUE, center = center,
     scale = FALSE
   )
-  if (verbose) {
-    varex <- sum(res$sdev[1:ncol]^2) / res$totalvar
-    tsmessage(
-      "PCA: ", ncol, " components explained ", formatC(varex * 100),
-      "% variance"
-    )
-  }
+  report_varex(res, verbose)
   if (ret_extra) {
     list(scores = res$x, rotation = res$rotation, center = res$center)
-  }
-  else {
+  } else {
     res$x
   }
 }
 
+report_varex <- function(res, verbose = FALSE) {
+  if (verbose) {
+    ncol <- ncol(res$rotation)
+    varex <- sum(res$sdev[1:ncol]^2) / res$totalvar
+    tsmessage(
+      "PCA: ",
+      ncol,
+      " components explained ",
+      formatC(varex * 100),
+      "% variance"
+    )
+  }
+}
+
+# This function taken from irlba and modified to use irlba::svdr rather
+# than irlba::irlba
+prcomp_rsvd <- function(x, n = 3, retx = TRUE, center = TRUE, scale. = FALSE,
+                        ...) {
+  a <- names(as.list(match.call()))
+  ans <- list(scale = scale.)
+  if ("tol" %in% a) {
+    warning("The `tol` truncation argument from `prcomp` is not supported by\n`prcomp_rsvd`. If specified, `tol` is passed to the `irlba` function to\ncontrol that algorithm's convergence tolerance. See `?prcomp_irlba` for help.")
+  }
+  if (is.data.frame(x)) {
+    x <- as.matrix(x)
+  }
+  args <- list(x = x, k = n)
+  if (is.logical(center)) {
+    if (center) {
+      args$center <- colMeans(x)
+    }
+  } else {
+    args$center <- center
+  }
+  if (is.logical(scale.)) {
+    if (is.numeric(args$center)) {
+      f <- function(i) {
+        sqrt(sum((x[, i] - args$center[i])^2) / (nrow(x) -
+          1L))
+      }
+      scale. <- vapply(seq(ncol(x)), f, pi, USE.NAMES = FALSE)
+      if (ans$scale) {
+        ans$totalvar <- ncol(x)
+      } else {
+        ans$totalvar <- sum(scale.^2)
+      }
+    } else {
+      if (ans$scale) {
+        scale. <- apply(x, 2L, function(v) {
+          sqrt(sum(v^2) / max(
+            1,
+            length(v) - 1L
+          ))
+        })
+        f <- function(i) {
+          sqrt(sum((x[, i] / scale.[i])^2) / (nrow(x) -
+            1L))
+        }
+        ans$totalvar <- sum(vapply(seq(ncol(x)), f, pi,
+          USE.NAMES = FALSE
+        )^2)
+      } else {
+        f <- function(i) sum(x[, i]^2) / (nrow(x) - 1L)
+        ans$totalvar <- sum(vapply(seq(ncol(x)), f, pi,
+          USE.NAMES = FALSE
+        ))
+      }
+    }
+    if (ans$scale) {
+      args$scale <- scale.
+    }
+  } else {
+    args$scale <- scale.
+    f <- function(i) {
+      sqrt(sum((x[, i] / scale.[i])^2) / (nrow(x) -
+        1L))
+    }
+    ans$totalvar <- sum(vapply(seq(ncol(x)), f, pi, USE.NAMES = FALSE))
+  }
+  if (!missing(...)) {
+    args <- c(args, list(...))
+  }
+
+  s <- do.call(irlba::svdr, args = args)
+  ans$sdev <- s$d / sqrt(max(1, nrow(x) - 1))
+  ans$rotation <- s$v
+  colnames(ans$rotation) <- paste("PC", seq(1, ncol(ans$rotation)),
+    sep = ""
+  )
+  ans$center <- args$center
+  if (retx) {
+    ans <- c(ans, list(x = sweep(s$u, 2, s$d, FUN = `*`)))
+    colnames(ans$x) <- paste("PC", seq(1, ncol(ans$rotation)),
+      sep = ""
+    )
+  }
+  class(ans) <- c("irlba_prcomp", "prcomp")
+  ans
+}
+
+irlba_svdr_scores <-
+  function(X,
+           ncol,
+           center = TRUE,
+           ret_extra = FALSE,
+           verbose = FALSE) {
+    # 5 iterations is the default for scikit-learn TruncatedSVD
+    res <- prcomp_rsvd(
+      X,
+      n = ncol,
+      retx = TRUE,
+      center = center,
+      scale. = FALSE,
+      it = 5
+    )
+    report_varex(res, verbose)
+    if (ret_extra) {
+      list(
+        scores = res$x,
+        rotation = res$rotation,
+        center = res$center
+      )
+    } else {
+      res$x
+    }
+  }
+
 init_is_spectral <- function(init) {
   res <- pmatch(tolower(init), c(
     "normlaplacian", "spectral", "laplacian",
-    "inormlaplacian", "ispectral"
+    "inormlaplacian", "ispectral", "agspectral",
+    "irlba_spectral", "irlba_laplacian"
   ))
   length(res) > 0 && !is.na(res)
 }
 
 rand_nbr_graph <- function(n_vertices, n_nbrs, val) {
-  nn_to_sparse(rand_nbr_idx(n_vertices, n_nbrs),
+  nng_to_sparse(rand_nbr_idx(n_vertices, n_nbrs),
     val = val,
     max_nbr_id = n_vertices
   )
